import pandas as pd
from typing import List, Dict
from get_pop.definitions import PATH_USA_POP, DIR_DATA
import logging


def parse_states(
    value_field: str,
    selected_values: List[Dict],
    selected_fields: List[Dict],
    *,
    field_cleaners: Dict = None,
    file_partial: str = None,
) -> None:

    # read
    df = pd.read_csv(PATH_USA_POP, encoding="ISO-8859-1")

    # filter - remove statewide population counts
    df = df[df["COUNTY"] != 0]

    # filter - include only selected values
    selected_values_names = [x["name"] for x in selected_values]
    df = df[df[value_field].isin(selected_values_names)]

    # option - clean value field
    if field_cleaners:
        for field in field_cleaners.keys():
            cleaner_func = field_cleaners[field]
            df = cleaner_func(df, field)

    # rename field lookuptable
    rename_schema = {}
    for field in selected_fields:
        input_name = field["input_name"]
        output_name = field["output_name"]
        rename_schema[input_name] = output_name

    # group by
    by_state = df.groupby(value_field)

    for name, group in by_state:
        logging.info(f"Processing: {name}")
        # get selected state dict
        selected_state = list(filter(lambda x: x["name"] == name, selected_values))[0]

        # truncate
        selected_fields_input = [x["input_name"] for x in selected_fields]
        group = group[selected_fields_input]

        # rename
        group = group.rename(columns=rename_schema)

        # option - special processor (special funcs for doing extra stuff to df)
        special_processors = selected_state.get("special_processors")
        if special_processors:
            for processor in special_processors:
                group = processor(group)

        # produce csv
        abbrv = selected_state["abbrv"]
        filename = (
            f"{abbrv}-{file_partial}-pop.csv" if file_partial else f"{abbrv}-pop.csv"
        )
        output_path = DIR_DATA / filename
        group.to_csv(output_path, index=False)



def clean_county(df, field):
    df[field] = df[field].str.replace(" County", "")
    return df


def merge_nyc_boroughs(df):
    """
    Combines the five buroughs to create NYC row
    """
    df_buroughs = df.loc[
        df["name"].isin(["Bronx", "Queens", "New York", "Kings", "Richmond"])
    ]
    nyc_pop = df_buroughs["population"].sum()
    return df.append(
        {"name": "New York City", "population": nyc_pop}, ignore_index=True
    )



import logging
import logging.config
import os
import yaml
from get_pop.definitions import DIR_LOGS_OUTPUT, PATH_LOGS_CONFIG


def logs_config(
    default_path=PATH_LOGS_CONFIG, default_level=logging.INFO, env_key="LOG_CFG"
):
    """
    Setup logging configuration
    @default_path: path of logging config file, eg. config/logging.yaml
    @default_level: default level that logs are logged at.
    @env_key: If env_key is detected among env variables, this will be used as path to config file
    """

    path = default_path

    # if file path is set using LOG_CFG env as path to config file
    value = os.getenv(env_key, None)
    if value:
        path = value

    # load config
    if os.path.exists(path):
        with open(path, "rt") as f:
            config = yaml.safe_load(f.read())  # convert yaml to dict
            # we get the path in config file then append it to our log directory
            for handler in config["handlers"]:
                if "FileHandler" in config["handlers"][handler]["class"]:
                    config["handlers"][handler]["filename"] = (
                        DIR_LOGS_OUTPUT / config["handlers"][handler]["filename"]
                    )
                    config["handlers"][handler]["filename"].parent.mkdir(
                        exist_ok=True, parents=True
                    )  # create parent
                    # directories of path if they don't exist
        logging.config.dictConfig(config)
    else:
        logging.basicConfig(level=default_level)



import logging
from dotenv import load_dotenv
from get_pop.definitions import DIR_DATA
from get_pop.logs.config.logging import logs_config
from get_pop.modules.helper.misc import delete_dir_contents
from get_pop.modules.helper.time import utc_now
from get_pop.modules.init.pandas_opts import pandas_opts


def init_program():
    # Load env vars
    load_dotenv()

    # init logging
    program_start_time = utc_now()
    timezone = program_start_time.tzinfo
    logs_config()
    logging.info(f"Initializing get_pop")

    # create or clean download dir
    if DIR_DATA.is_dir():
        # delete files from previous run
        delete_dir_contents(DIR_DATA)
    else:
        logging.info("Data directory doesn't exist - building")
        DIR_DATA.mkdir()

    pandas_opts()

    return program_start_time



import pandas as pd


def pandas_opts():
    # Sets desired pandas options
    pd.set_option("display.max_columns", 20)
    pd.set_option("display.width", 2000)
    pd.set_option("display.max_rows", 2000)



from datetime import datetime
import pytz
import logging
import iso8601

def utc_now():
    return pytz.utc.localize(datetime.utcnow())


def convert_utc_to_est(utc):
    return utc.astimezone(pytz.timezone("US/Eastern"))


def format_datetime_eastern(datetime_obj):
    # Formats a datetime obj into a pretty string in US/Eastern timezone
    est_time = convert_utc_to_est(datetime_obj)
    return est_time.strftime("%b %-d %Y, %I:%M %p")


def convert_iso_to_datetime(iso_str):
    # Converts ISO string into datetime obj
    logging.info(f"Converting from ISO to datetime obj...")
    datetime_obj = iso8601.parse_date(iso_str)
    assert (isinstance(datetime_obj, datetime)), f"Failed to convert date into " \
                                                                 f"datetime obj"
    logging.info(f"Datetime obj: {iso_str}")
    return datetime_obj


import os
import shutil


def delete_dir_contents(folder):
    for filename in os.listdir(folder):
        file_path = os.path.join(folder, filename)
        try:
            if os.path.isfile(file_path) or os.path.islink(file_path):
                os.unlink(file_path)
            elif os.path.isdir(file_path):
                shutil.rmtree(file_path)
        except Exception as e:
            print("Failed to delete %s. Reason: %s" % (file_path, e))



from get_pop.modules.init.init_program import init_program
from get_pop.modules.parse.parse import parse_states
import click
import logging
from get_pop.static.constants import (
    state_index,
    value_field,
    selected_fields,
    field_cleaners,
)

clean_states = [x["abbrv"] for x in state_index]


@click.command()
@click.argument("states", nargs=-1, type=click.Choice(clean_states))
def main(states):

    # init
    init_program()

    # actions
    states = list(states)
    logging.info(f"Selected states: {states}")
    selected_states = []
    for state in states:
        index_item = [x for x in state_index if x["abbrv"].lower() in state.lower()][0]
        selected_states.append(index_item)
    parse_states(
        value_field=value_field,
        selected_values=selected_states,
        selected_fields=selected_fields,
        field_cleaners=field_cleaners,
        file_partial="county",
    )



from setuptools import setup, find_packages

requires = ["pyyaml", "python-dotenv", "pandas", "iso8601", "click"]

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="get_pop",
    version="1.7",
    author="DSR",
    description="Command line tool to generates CSVs of population data for specified US states.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
    ],
    keywords=["population csv geography usa counties"],
    # list folders, not files
    packages=find_packages(exclude=["docs", "tests*"]),
    install_requires=requires,
    scripts=["get_pop/bin/getpop"],
    package_data={"get_pop": ["static/*"]},
)
